var boxes = [{"name": "Float", "color": "#598053", "inputs": [], "outputs": [{"name": "Float", "types": ["float", "int"], "color": "#96F632", "value": 0.0}], "parameters": [], "primitive": "Float", "gist": "Variable", "value": 0}, {"name": "Int", "color": "#598053", "inputs": [], "outputs": [{"name": "Int", "types": ["int"], "color": "#96F632", "value": 0}], "parameters": [], "primitive": "Int", "gist": "Variable", "value": 0}, {"name": "String", "color": "#598053", "inputs": [], "outputs": [{"name": "Str", "types": ["str"], "color": "#96F632", "value": ""}], "parameters": [], "primitive": "Str", "gist": "Variable", "value": ""}, {"name": "Cast to Int", "color": "#111111", "inputs": [{"name": "Object", "types": ["int", "float", "str"], "color": "#96F632", "value": 0}], "outputs": [{"name": "Int", "types": ["int"], "color": "#96F632", "value": 0}], "parameters": [], "primitive": "ToInt", "gist": "Cast"}, {"name": "Cast to Str", "color": "#111111", "inputs": [{"name": "Object", "types": ["int", "float", "str", "dict", "list"], "color": "#96F632", "value": {}}], "outputs": [{"name": "Str", "types": ["str"], "color": "#96F632", "value": ""}], "parameters": [], "primitive": "ToStr", "gist": "Cast"}, {"name": "Cast to JSON", "color": "#111111", "inputs": [{"name": "Object", "types": ["str"], "color": "#96F632", "value": ""}], "outputs": [{"name": "JSON", "types": ["dict"], "color": "#96F632", "value": {}}], "parameters": [], "primitive": "ToJson", "gist": "Cast"}, {"name": "Write to file", "color": "#BA9303", "inputs": [{"name": "content", "types": ["int", "float", "str", "dict", "list"], "color": "#96F632", "value": {}}], "outputs": [], "parameters": [{"name": "filename", "types": ["str"], "color": "#96F632", "value": ""}], "primitive": "ToFile", "gist": "Box"}, {"name": "Add", "color": "#49718B", "inputs": [{"name": "x", "types": ["float", "int"], "color": "#96F632", "value": 0.0}, {"name": "y", "types": ["float", "int"], "color": "#96F632", "value": 0.0}], "outputs": [{"name": "x + y", "types": ["float", "int"], "color": "#96F632", "value": 0.0}], "parameters": [], "primitive": "Add", "gist": "Function"}, {"name": "Substract", "color": "#49718B", "inputs": [{"name": "x", "types": ["float", "int"], "color": "#96F632", "value": 0.0}, {"name": "y", "types": ["float", "int"], "color": "#96F632", "value": 0.0}], "outputs": [{"name": "x - y", "types": ["float", "int"], "color": "#96F632", "value": 0.0}], "parameters": [], "primitive": "Substract", "gist": "Function"}, {"name": "Get item", "color": "#49718B", "inputs": [{"name": "json", "types": ["str"], "color": "#96F632", "value": ""}], "outputs": [{"name": "value", "types": ["float", "int"], "color": "#96F632", "value": 0.0}, {"name": "KeyError", "types": ["float", "int"], "color": "#96F632", "value": 0.0}], "parameters": [{"name": "key", "types": ["str"], "color": "#96F632", "value": ""}], "primitive": "GetItem", "gist": "Function"}, {"name": "Environment variable", "color": "#598053", "inputs": [], "outputs": [{"name": "Str", "types": ["str"], "color": "#96F632", "value": ""}], "parameters": [], "primitive": "Environment", "gist": "Variable", "value": "PATH"}, {"name": "Print", "color": "#BA9303", "inputs": [{"name": "text", "types": ["int", "float", "str", "dict", "list"], "color": "#96F632", "value": {}}], "outputs": [], "parameters": [], "primitive": "Print", "gist": "Box"}, {"name": "GET Request", "color": "#49718B", "inputs": [{"name": "url", "types": ["str"], "color": "#96F632", "value": ""}], "outputs": [{"name": "text", "types": ["str"], "color": "#96F632", "value": ""}, {"name": "status code", "types": ["int"], "color": "#96F632", "value": 0}], "parameters": [], "primitive": "GetRequest", "gist": "Function"}];